"""Asfsmd client based on smart-open."""

import zipfile
import contextlib
from typing import Optional

import smart_open

from .common import AbstractClient, Auth, Url


class SmartOpenClient(AbstractClient):
    def __init__(self, auth: Auth, block_size: Optional[int] = None):
        self.client_kwargs = None
        if auth is not None:
            self.client_kwargs["user"] = auth.user
            self.client_kwargs["password"] = auth.pwd
        if block_size is not None:
            self.client_kwargs["buffer_size"] = block_size

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        pass

    @contextlib.contextmanager
    def open_zip_archive(self, url: Url) -> zipfile.ZipFile:
        with smart_open.open(url, "rb", **self.client_kwargs) as fd:
            with zipfile.ZipFile(fd) as zf:
                yield zf


Client = SmartOpenClient



"""Common constants and types."""

import os
import abc
from typing import NamedTuple, Union


MB = 1024 * 1024
BLOCKSIZE = 16 * MB  # 16MB (64MB is a better choice to download data)


PathType = Union[str, bytes, os.PathLike]
Url = str


class Auth(NamedTuple):
    user: str
    pwd: str


class AbstractClient(abc.ABC):
    @abc.abstractmethod
    def open_zip_archive(self, url: Url):
        pass
